/** @file modbuscli.c MODBUS/UDP client sample for the MSXE1711 and MSXE1721
*
* FC16 queries are for setting things, FC3 for fetching values
*
* This code doesn't handle timeouts.
*/

#ifdef WIN32
	#include "../../Interface_Library/msxe171x_modbus_clientlib.h"
#else
	#include <msxe171x_modbus_clientlib.h>
#endif
#include <addidata_network.h>

#ifdef WIN32
#include <windows.h>
#define sleep Sleep
#else
#include <unistd.h>
#endif // WIN32

#include <modbusclient.h>
#include <commonclient.h>

#include <addidata_network.h>

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFSinCosInit(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;


	uint32_t ulMode         = 1;
	uint32_t ulSignalPeriod = 2;
	uint32_t ulResolution   = 128;
	uint32_t ulValueFormat  = 0;
	uint32_t ulOption01     = 0;
	uint32_t ulOption02     = 0;
	uint32_t ulOption03     = 0;
	uint32_t ulOption04     = 0;

	//Remote function call
	ret = modbus_call_MSXE17xx__MFSinCosInit(modbus, ulMFModuleIndex,ulMode, ulSignalPeriod, ulResolution, ulValueFormat, ulOption01, ulOption02, ulOption03, ulOption04, &CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFSinCosInit" );
}
//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFSinCosInitAbsPos(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;


	uint32_t ulSignalPeriod     = 2;
	uint32_t ulNominalIncrement = 1000;
	uint32_t ulResolution       = 128;
	uint32_t ulValueFormat      = 0;
	uint32_t ulInvert           = 0;
	uint32_t ulOption01         = 0;
	uint32_t ulOption02         = 0;
	uint32_t ulOption03         = 0;
	uint32_t ulOption04         = 0;

	//Remote function call
	ret = modbus_call_MSXE17xx__MFSinCosInitAbsPos(modbus, ulMFModuleIndex,ulSignalPeriod, ulNominalIncrement, ulResolution, ulValueFormat, ulInvert, ulOption01, ulOption02, ulOption03, ulOption04, &CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFSinCosInitAbsPos" );
}


//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFSinCosRead(struct modbus * modbus, uint32_t ulMFModuleIndex, uint32_t *pulValue, uint32_t *pulMeasureError)
{
	//variables declaration
	struct MSXE17xx__MFSinCosRead0_parameters_t Response0;
	struct MSXE17xx__MFSinCosRead1_parameters_t Response1;
	struct MSXE17xx__MFSinCosRead2_parameters_t Response2;
	struct MSXE17xx__MFSinCosRead3_parameters_t Response3;
	int ret = 0;

	switch (ulMFModuleIndex)
	{
		case 0:
			memset(&Response0,0,sizeof(Response0));

			ret = modbus_call_MSXE17xx__MFSinCosRead0(modbus,&Response0);

			if (handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE17xx__MFSinCosRead0"))
			{
				return -1;
			}

			*pulValue        = Response0.ulValue;
			*pulMeasureError = Response0.ulMeasureError;
			break;

		case 1:
			memset(&Response1,0,sizeof(Response1));

			ret = modbus_call_MSXE17xx__MFSinCosRead1(modbus,&Response1);

			if (handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE17xx__MFSinCosRead1"))
			{
				return -1;
			}

			*pulValue        = Response1.ulValue;
			*pulMeasureError = Response1.ulMeasureError;
			break;

		case 2:
			memset(&Response2,0,sizeof(Response2));

			ret = modbus_call_MSXE17xx__MFSinCosRead2(modbus,&Response2);

			if (handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE17xx__MFSinCosRead2"))
			{
				return -1;
			}

			*pulValue        = Response2.ulValue;
			*pulMeasureError = Response2.ulMeasureError;
			break;

		case 3:
			memset(&Response3,0,sizeof(Response3));

			ret = modbus_call_MSXE17xx__MFSinCosRead3(modbus,&Response3);

			if (handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE17xx__MFSinCosRead3"))
			{
				return -1;
			}

			*pulValue        = Response3.ulValue;
			*pulMeasureError = Response3.ulMeasureError;
			break;
	}


	printf("Counter/position: %d\n",*pulValue);
	printf("Error:            %u\n",*pulMeasureError);

	return 0;
}
//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFSinCosReadAbsPos(struct modbus * modbus, uint32_t ulMFModuleIndex, uint32_t *pulValue, uint32_t *pulAbsPos, uint32_t *pulAbsPosStatus, uint32_t *pulMeasureError)
{
	//variables declaration
	struct MSXE17xx__MFSinCosReadAbsPos0_parameters_t Response0;
	struct MSXE17xx__MFSinCosReadAbsPos1_parameters_t Response1;
	struct MSXE17xx__MFSinCosReadAbsPos2_parameters_t Response2;
	struct MSXE17xx__MFSinCosReadAbsPos3_parameters_t Response3;
	int ret = 0;

	switch (ulMFModuleIndex)
	{
		case 0:
			memset(&Response0,0,sizeof(Response0));

			ret = modbus_call_MSXE17xx__MFSinCosReadAbsPos0(modbus,&Response0);

			if (handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE17xx__MFSinCosReadAbsPos0"))
			{
				return -1;
			}

			*pulValue        = Response0.ulValue;
			*pulAbsPos       = Response0.ulAbsPos;
			*pulAbsPosStatus = Response0.ulAbsPosStatus;
			*pulMeasureError = Response0.ulMeasureError;
			break;

		case 1:
			memset(&Response1,0,sizeof(Response1));

			ret = modbus_call_MSXE17xx__MFSinCosReadAbsPos1(modbus,&Response1);

			if (handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE17xx__MFSinCosReadAbsPos1"))
			{
				return -1;
			}

			*pulValue        = Response1.ulValue;
			*pulAbsPos       = Response1.ulAbsPos;
			*pulAbsPosStatus = Response1.ulAbsPosStatus;
			*pulMeasureError = Response1.ulMeasureError;
			break;

		case 2:
			memset(&Response2,0,sizeof(Response2));

			ret = modbus_call_MSXE17xx__MFSinCosReadAbsPos2(modbus,&Response2);

			if (handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE17xx__MFSinCosReadAbsPos2"))
			{
				return -1;
			}

			*pulValue        = Response2.ulValue;
			*pulAbsPos       = Response2.ulAbsPos;
			*pulAbsPosStatus = Response2.ulAbsPosStatus;
			*pulMeasureError = Response2.ulMeasureError;
			break;

		case 3:
			memset(&Response3,0,sizeof(Response3));

			ret = modbus_call_MSXE17xx__MFSinCosReadAbsPos3(modbus,&Response3);

			if (handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE17xx__MFSinCosReadAbsPos3"))
			{
				return -1;
			}

			*pulValue        = Response3.ulValue;
			*pulAbsPos       = Response3.ulAbsPos;
			*pulAbsPosStatus = Response3.ulAbsPosStatus;
			*pulMeasureError = Response3.ulMeasureError;
			break;
	}


	printf("Counter/position:                %d\n",*pulValue);
	printf("Counter/absolut position:        %d\n",*pulAbsPos);
	printf("Counter absolut position status: %d\n",*pulAbsPosStatus);
	printf("Error:                           %u\n",*pulMeasureError);

	return 0;
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFSinCosClear(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulOption01 = 0;
	uint32_t ulOption02 = 0;

	//Remote function call
	ret = modbus_call_MSXE17xx__MFSinCosClear(modbus, ulMFModuleIndex, ulOption01, ulOption02, &CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFSinCosClear" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFSinCosRelease(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulOption01 = 0;
	uint32_t ulOption02 = 0;

	//Remote function call
	ret = modbus_call_MSXE17xx__MFSinCosRelease(modbus, ulMFModuleIndex, ulOption01, ulOption02, &CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFSinCosRelease" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFSinCosInitAndEnableLatch(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulLatchSource 		= 0; // Index
	uint32_t ulCondition   		= 0;
	uint32_t ulAutoMode    		= 1;
	uint32_t ulLatchDigValue    = 0; //Option01 used for LatchDigValue: HO 28.06.12
	uint32_t ulOption02    		= 0;

	//Remote function call
	ret = modbus_call_MSXE17xx__MFSinCosInitAndEnableLatch(modbus, ulMFModuleIndex, ulLatchSource, ulCondition, ulAutoMode, ulLatchDigValue, ulOption02, &CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFSinCosInitAndEnableLatch" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFSinCosDisableAndReleaseLatch(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulLatchSource = 0; // Index
	uint32_t ulOption01    = 0;

	//Remote function call
	ret = modbus_call_MSXE17xx__MFSinCosDisableAndReleaseLatch(modbus, ulMFModuleIndex, ulLatchSource, ulOption01, &CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFSinCosDisableAndReleaseLatch" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFSinCosInitAndEnableClear(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulClearSource = 1; // Hardware trigger
	uint32_t ulCondition   = 0;
	uint32_t ulAutoMode    = 1;
	uint32_t ulOption01    = 0;
	uint32_t ulOption02    = 0;

	//Remote function call
	ret = modbus_call_MSXE17xx__MFSinCosInitAndEnableClear(modbus, ulMFModuleIndex, ulClearSource, ulCondition, ulAutoMode, ulOption01, ulOption02, &CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFSinCosInitAndEnableClear" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFSinCosDisableAndReleaseClear(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulClearSource = 1; // Hardware trigger
	uint32_t ulOption01    = 0;

	//Remote function call
	ret = modbus_call_MSXE17xx__MFSinCosDisableAndReleaseClear(modbus, ulMFModuleIndex, ulClearSource, ulOption01, &CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFSinCosDisableAndReleaseClear" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFSinCosInitAndEnableCompareLogic(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	double   dCompareValue = 1;
	uint32_t *pulCompareValue = (uint32_t *) &dCompareValue;

	uint32_t ulValueLow       = pulCompareValue[0];
	uint32_t ulValueHigh      = pulCompareValue[1];
	uint32_t ulMode           = 1; // Modulo
	uint32_t ulSynchroTrigger = 0;
	uint32_t ulLatchDigValue  = 0; //Option01 used for LatchDigValue: HO 28.06.12
	uint32_t ulOption02       = 0;

	//Remote function call
	ret = modbus_call_MSXE17xx__MFSinCosInitAndEnableCompareLogic(modbus, ulMFModuleIndex, ulValueLow, ulValueHigh, ulMode, ulSynchroTrigger, ulLatchDigValue, ulOption02, &CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFSinCosInitAndEnableCompareLogic" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFSinCosDisableAndReleaseCompareLogic(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	//Remote function call
	ret = modbus_call_MSXE17xx__MFSinCosDisableAndReleaseCompareLogic(modbus, ulMFModuleIndex, &CommandStatus);


	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFSinCosDisableAndReleaseCompareLogic" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFSinCosInitHardwareTrigger(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulEdgeSelection  = 1; // Rising front
	uint32_t ulCount          = 1;
	uint32_t ulOption01       = 0;
	uint32_t ulOption02       = 0;

	//Remote function call
	ret = modbus_call_MSXE17xx__MFSinCosInitHardwareTrigger(modbus, ulMFModuleIndex, ulEdgeSelection, ulCount, ulOption01, ulOption02, &CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFSinCosInitHardwareTrigger" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFSinCosReleaseHardwareTrigger(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulOption01 = 0;

	//Remote function call
	ret = modbus_call_MSXE17xx__MFSinCosReleaseHardwareTrigger(modbus, ulMFModuleIndex, ulOption01, &CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFSinCosReleaseHardwareTrigger" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFSinCosInitIndex(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulEdgeSelection  = 1; // Rising front
	uint32_t ulOption01       = 0;
	uint32_t ulOption02       = 0;

	//Remote function call
	ret = modbus_call_MSXE17xx__MFSinCosInitIndex(modbus, ulMFModuleIndex, ulEdgeSelection, ulOption01, ulOption02, &CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFSinCosInitIndex" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFSinCosReleaseIndex(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulOption01 = 0;

	//Remote function call
	ret = modbus_call_MSXE17xx__MFSinCosReleaseIndex(modbus, ulMFModuleIndex, ulOption01, &CommandStatus);


	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFSinCosReleaseIndex" );
}



//---------------------------------------------------------------------------------
static void usage(char* name)
{
	printf("%s ADDRESS tcp|udp [Modul index: 0 or 1 or 2 or 3]\n",name);
}

//---------------------------------------------------------------------------------
int main(int argc, char** argv)
{
	int           i               = 0;
	uint32_t      ulMFModuleIndex = 0;
	uint32_t      ulValue         = 0;
	uint32_t      ulAbsPos        = 0;
	uint32_t      ulAbsPosStatus  = 0;
	uint32_t      ulMeasureError  = 0;
	int           contype         = 0;
	struct modbus modbus;



	if (argc < 3)
	{
		usage(argv[0]);
		exit(1);
	}

	if (0 == strcmp(argv[2],"tcp"))
	{
		contype=SOCK_STREAM;
	}
	else if(0 == strcmp(argv[2],"udp"))
	{
		contype=SOCK_DGRAM;
	}
	else
	{
		usage(argv[0]);
		exit(1);
	}

	if (argc > 3)
	{
		ulMFModuleIndex = (uint32_t) (argv[3][0] - '0');
		if (ulMFModuleIndex > 3)
		{
			usage(argv[0]);
			exit(1);
		}
	}

	modbus_init(&modbus);

	switch( modbus_connect(&modbus, argv[1], contype, 0) )
	{
		case 0: /* success */
			break;
		case -1: /* system error */
			addidata_network_perror("modbus_connect");
			exit(1);
		case -2: /* parameter is incorrect */
			printf("modbus_connect: incorrect parameter\n");
			exit(1);
		case -3: /* incorrect address */
			printf("incorrect address\n");
			exit(1);
	}

	assert (sample_MSXE17xx__MFSinCosInit(&modbus, ulMFModuleIndex) == 0);
	assert (sample_MSXE17xx__MFSinCosClear(&modbus, ulMFModuleIndex) == 0);
	assert (sample_MSXE17xx__MFSinCosInitIndex(&modbus, ulMFModuleIndex) == 0);
	assert (sample_MSXE17xx__MFSinCosInitHardwareTrigger(&modbus, ulMFModuleIndex) == 0);

	for(i=0;i<500;i++)
	{
		assert (sample_MSXE17xx__MFSinCosRead(&modbus, ulMFModuleIndex, &ulValue, &ulMeasureError) == 0);
		sleep (100);
	}

	assert (sample_MSXE17xx__MFSinCosInitAndEnableLatch(&modbus, ulMFModuleIndex) == 0);
	sleep (2000);
	assert (sample_MSXE17xx__MFSinCosDisableAndReleaseLatch(&modbus, ulMFModuleIndex) == 0);

	assert (sample_MSXE17xx__MFSinCosInitAndEnableClear(&modbus, ulMFModuleIndex) == 0);
	sleep (2000);
	assert (sample_MSXE17xx__MFSinCosDisableAndReleaseClear(&modbus, ulMFModuleIndex) == 0);

	assert (sample_MSXE17xx__MFSinCosInitAndEnableCompareLogic(&modbus, ulMFModuleIndex) == 0);
	sleep (2000);
	assert (sample_MSXE17xx__MFSinCosDisableAndReleaseCompareLogic(&modbus, ulMFModuleIndex) == 0);

	assert (sample_MSXE17xx__MFSinCosReleaseHardwareTrigger(&modbus, ulMFModuleIndex) == 0);
	assert (sample_MSXE17xx__MFSinCosReleaseIndex(&modbus, ulMFModuleIndex) == 0);
	assert (sample_MSXE17xx__MFSinCosRelease(&modbus, ulMFModuleIndex) == 0);

	assert (sample_MSXE17xx__MFSinCosInitAbsPos(&modbus, ulMFModuleIndex) == 0);

	for(i=0;i<500;i++)
	{
		assert (sample_MSXE17xx__MFSinCosReadAbsPos(&modbus, ulMFModuleIndex, &ulValue, &ulAbsPos, &ulAbsPosStatus, &ulMeasureError) == 0);
		sleep (100);
	}

	assert (sample_MSXE17xx__MFSinCosRelease(&modbus, ulMFModuleIndex) == 0);

	return 0;
}
